
#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <ArduinoOTA.h>

// For reference:
// EIGHT
// D0/GPIO16      = 16;
// D1/GPIO5       = 5;
// D2/GPIO4       = 0;
// D3/GPIO0       = 4;

// ZERO
// D4/GPIO2       = 2;
// D5/GPIO14      = 14;
// D6/GPIO12      = 13;
// D7/GPIO13      = 12;

// NINE
// D8/GPIO15      = 15;
// RX/GPIO3       = 3;
// GPIO10       = ??;
// GPIO9      = 9;

// Eight
//#define EIGHT_WHITE_LED   16
//#define EIGHT_BLUE_LED    5
//#define EIGHT_RED_LED     0
//#define EIGHT_GREEN_LED   4

// Zero
//#define ZERO_WHITE_LED   2
//#define ZERO_BLUE_LED    14
//#define ZERO_RED_LED       13
//#define ZERO_GREEN_LED   12

// Nine
#define WHITE_LED     15
#define BLUE_LED      3
#define RED_LED       13 // HERE
#define GREEN_LED     1

int maxBrightness = 1024;
int offBrightness = 0;

const char* ssid     = "ostmo";
const char* password = "stikaa11";
int serverPort = 80;
int status = WL_IDLE_STATUS;
IPAddress ip;

WiFiServer server( serverPort  );

int currentR = 0;
int currentG = 0;
int currentB = 0;
int currentW = 0;


void setup() 
{
  //Serial.begin( 115200 );

//  pinMode( GREEN_LED, FUNCTION_0 );
//  pinMode( RED_LED, FUNCTION_0 );
//  pinMode( BLUE_LED, FUNCTION_0 );
//  pinMode( WHITE_LED, FUNCTION_0 );
  
  
  pinMode( WHITE_LED, OUTPUT );
  pinMode( BLUE_LED, OUTPUT );
  pinMode( RED_LED, OUTPUT );
  pinMode( GREEN_LED, FUNCTION_3 );
  

  connectToWifi();
  initOta();
  server.begin(); // Start the API server/service

  //TurnOn();

}

void turnOn() 
{ 
  setColorToLevel( "all", maxBrightness );
}

void turnOff() 
{
  setColorToLevel( "all", offBrightness );
}


void setColorToLevel( String color, int level ) 
{ 
  if( color == "red" )
  {
    analogWrite( RED_LED, level );
    currentR = level;
  }
  if( color == "green" )
  {
    analogWrite( GREEN_LED, level );
    currentG = level;
  }
  if( color == "blue" )
  {
    analogWrite( BLUE_LED, level );
    currentB = level;
  }
  if( color == "white" )
  {
    analogWrite( WHITE_LED, level );
    currentW = level;
  }
  if( color == "all" )
  {
    analogWrite( RED_LED, level );
    analogWrite( GREEN_LED, level );
    analogWrite( BLUE_LED, level );
    analogWrite( WHITE_LED, level );
    currentR = level;
    currentG = level;
    currentB = level;
    currentW = level;
  }
}

void loop() 
{
  ArduinoOTA.handle();
  nodeServer();
}

void connectToWifi()
{
  //Serial.print( "Connecting to " );
  //Serial.println( ssid );

  WiFi.mode( WIFI_STA );
  WiFi.begin( ssid, password );

  while( WiFi.status() != WL_CONNECTED )
  {
      delay( 500 );
      //Serial.print( "." );
  }
  //Serial.println( "." );
  //Serial.println( "Connected" );
  printWifiStatus();
}

void printWifiStatus() 
{
  // print the SSID of the network you're attached to:
//  Serial.print( "SSID: " );
//  Serial.println( WiFi.SSID() );

  // print your WiFi shield's IP address:
  ip = WiFi.localIP();
  //Serial.print( "IP Address: " );
  //Serial.println( ip );

  // print the received signal strength:
//  long rssi = WiFi.RSSI();
//  Serial.print( "Signal strength (RSSI):" );
//  Serial.print( rssi );
//  Serial.println (" dBm" );
}

void htmlHeader( WiFiClient client )
{
    client.println( "HTTP/1.1 200 OK" );
    client.println ("Content-Type: text/html" );
    client.println( "Connection: close" );  // the connection will be closed after completion of the response
    client.println( "Cache-Control: no-store" );
    client.println();
    client.println( "<!DOCTYPE HTML>" );
    client.println( "<html>" );
}

void htmlFooter( WiFiClient client )
{
    client.println( "</html>" );
}

void initOta()
{
  ArduinoOTA.onStart( []() 
  {
    String type;
    if( ArduinoOTA.getCommand() == U_FLASH )
    {
      type = "sketch";
    }
    else // U_SPIFFS
    {
      type = "filesystem";
    }

    // NOTE: if updating SPIFFS this would be the place to unmount SPIFFS using SPIFFS.end()
    //Serial.println( "Start updating " + type );
  });
  
  ArduinoOTA.onEnd( []() 
  {
    //Serial.println( "\nEnd" );
  });
  
  ArduinoOTA.onProgress( [](unsigned int progress, unsigned int total ) 
  {
    //Serial.printf( "Progress: %u%%\r", (progress / (total / 100)) );
  });
  
  ArduinoOTA.onError( []( ota_error_t error ) 
  {
    //Serial.printf( "Error[%u]: ", error );
//    if( error == OTA_AUTH_ERROR) { Serial.println( "Auth Failed" ); }
//    else if(error == OTA_BEGIN_ERROR) { Serial.println("Begin Failed"); }
//    else if(error == OTA_CONNECT_ERROR) { Serial.println("Connect Failed"); }
//    else if(error == OTA_RECEIVE_ERROR) { Serial.println("Receive Failed"); }
//    else if(error == OTA_END_ERROR) { Serial.println("End Failed"); }
  });
  ArduinoOTA.begin();
}

int getLevelForColor( String color )
{
  if( color == "red" )
  {
    return currentR;
  }
  else if( color == "green" )
  {
    return currentG;
  }
  else if( color == "blue" )
  {
    return currentB;
  }
  else if( color == "white" )
  {
    return currentW;
  }
  else
  {
    return 0;
  }
}

float getRatioForColor( String color )
{
  int  currentValue = getLevelForColor( color );
  return (float) ((float)currentValue / (float)1024);
}

// A tiny API server for a farm master node
void nodeServer()
{
   WiFiClient client = server.available();
   if( client )
   {
      String requestString; 
      boolean currentLineIsBlank = true;
      while( client.connected() )
      {
        if( client.available() )
        {
          char c = client.read();
          requestString += c;

          // request is complete, parse it
          if( c == '\n' && currentLineIsBlank )
          {
              htmlHeader( client );

              // Over-the-air (433 mHz) transmissions
              if( requestString.indexOf( "?action=otasend" ) > 0 )
              {
                int otaCode = parseOtaRequest( requestString );
                //rcSwitchSend( otaCode );
                client.println( "ACK" );
              }
              else if( requestString.indexOf( "?action=on" ) > 0 )
              {
                turnOn();
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=off" ) > 0 )
              {
                turnOff();
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=ron" ) > 0 )
              {
                setColorToLevel( "red", maxBrightness );
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=roff" ) > 0 )
              {
                setColorToLevel( "red", 0 );
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=gon" ) > 0 )
              {
                setColorToLevel( "green", maxBrightness );
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=goff" ) > 0 )
              {
                setColorToLevel( "green", 0 );
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=bon" ) > 0 )
              {
                setColorToLevel( "blue", maxBrightness );
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=boff" ) > 0 )
              {
                setColorToLevel( "blue", 0 );
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=won" ) > 0 )
              {
                setColorToLevel( "white", maxBrightness );
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=woff" ) > 0 )
              {
                setColorToLevel( "white", 0 );
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=25" ) > 0 )
              {
                setColorToLevel( "all", 256 );
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=50" ) > 0 )
              {
                setColorToLevel( "all", 512 );
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=75" ) > 0 )
              {
                setColorToLevel( "all", 768 );
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=values" ) > 0 )
              {
                client.println( getValues() );
              }
              else if( requestString.indexOf( "?action=id" ) > 0 )
              {
                byte mac[6];
                WiFi.macAddress( mac );
                // Using client.write multiple times caused scrolling output. Building a string causes instant output on completion, which is preferable
//                String output = nodeType;
//                output += "|";
                String output = String( mac[0], HEX );
                output +=  ":" ;
                output += String( mac[1], HEX );
                output +=  ":" ;
                output += String( mac[2], HEX );
                output +=  ":" ;
                output += String( mac[3], HEX );
                output +=  ":" ;
                output += String( mac[4], HEX );
                output +=  ":" ;
                output += String( mac[5], HEX );
                output += "|";
                output += WiFi.localIP();
//                output += "|";
//                output += nodeVersion;
                client.println( output );
              }
              else
              {
                client.println( "ERR" );
              }
              htmlFooter( client );
              break;
          } // if( c == '\n' && currentLineIsBlank )
          if( c == '\n' ) 
          {
            currentLineIsBlank = true;
          } 
          else if( c != '\r' ) 
          {
            // you've gotten a character on the current line
            currentLineIsBlank = false;
          }
          
        } // if( client.available() )
      } // while( client.connected() )

    // give the web browser time to receive the data
    delay( 1 );

    // close the connection:
    client.stop();
   } // if( client )
} // void nodeServer()

int parseOtaRequest( String requestString )
{
  char *line;
  char *token;
  char *tokenOut;
  char *finalToken;
  char *currentString = (char *)requestString.c_str();
  String myString = String( currentString );
  
  line = strtok( currentString, "\n" );
  while( line != NULL )
  {
    if( String( line ).indexOf( "?action=otasend" ) > 0 )
    {
      token = tokenOut = strtok( line, "-" );
      while( token != NULL )
      {
        token = strtok( NULL, "-") ;
        if( token != NULL )
        {
          tokenOut = token;
        } 
      }
      break;
    }
    line = strtok( NULL, "\n") ;
  }
  
  if( tokenOut != NULL )
  {
     finalToken = strtok( tokenOut, " " );
  }

  return String( finalToken ).toInt();
}

String getValues()
{
  String output = "<p>Red value: ";
                output += getLevelForColor( "red" );
                output += "<br>";
                output += "Red ratio: ";
                output += getRatioForColor( "red" );
                output += "&nbsp;</p>";

                output += "<p>Green value: ";
                output += getLevelForColor( "green" );
                output += "<br>";
                output += "Green ratio: ";
                output += getRatioForColor( "green" );
                output += "&nbsp;</p>";

                output += "<p>Blue value: ";
                output += getLevelForColor( "blue" );
                output += "<br>";
                output += "Blue ratio: ";
                output += getRatioForColor( "blue" );
                output += "&nbsp;</p>";

                output += "<p>White value: ";
                output += getLevelForColor( "white" );
                output += "<br>";
                output += "White ratio: ";
                output += getRatioForColor( "white" );
                output += "&nbsp;</p>";

  return output;
}

